# bd_pod_extentions

![](https://bupter-test.oss-cn-shanghai.aliyuncs.com/IMG_0205.PNG)

## 概述

bd_pod_extentions 作为组件化整体方案中的 Native Component 是在 cocoapods 基础上做了一层自定义扩展。

## 功能

- [x] 一键配置各业务方组件库
- [x] 全自动二进制化方案
- [x] 支持单个 Pod 源码与二进制版本切换
- [x] 支持全量二进制版本与源码版本切换
- [x] 依赖解析加速
- [x] CI 定制化构建加速
- [x] Pods 废弃功能通知
- [x] Slardar平台打通
- [x] 二进制调试
- [x] 组件混淆

**_Note:_** **整个构建过程需要在内网环境**

## 使用方式

### 安装

```bash
sudo gem install bd_pod_extentions
```

### 自定义解析指令

在二进制化方案设计之初，考虑到需要用最低成本最简单方式来做到 **_源码_** 与 **_二进制_** 版本之间的切换。因此最终定位到重构解析指令 **_pod_** ，这里我们写了一个 Cocoapods 插件——**_bd_pod_extentions_**,提供了若干种自定义的指令来完成 cocoapods 的扩展

```ruby
bd_use_app
bytedanceAnalyzeSpeed
bd_use_source!
bd_use_binary!
pod_binary
pod_source
plugin 'cocoapods-bytedance-transform'
plugin 'cocoapods-bytedance-Slardar'
plugin 'cocoapods-bytedance-DebugBinary'
plugin 'cocoapods-bytedance-PlatformRecord'
bytedanceRemoveAllConfuse
bd_enable_DevelopMode
showCode!
:confuse
```

- [1] bd_use_app

  bd_use_app 指令用于指定业务线组件资源，选取合适的 spec source，比如，使用头条业务可以用下面来指令

  ```ruby
  bd_use_app("toutiao")
  ```

  比如使用抖音的组件资源可以使用下面指令

  ```ruby
  bd_use_app("douyin")
  ```

  其他可用的 key word：

  ```
  toutiao
  douyin
  wenda
  xigua
  thirdParty
  public
  ......
  ```

- [2] bytedanceAnalyzeSpeed(enable=true,type = {:project_type => 'objective-c'})

  bytedanceAnalyzeSpeed 是针对**头条项目**开发的**依赖加速**功能，由于头条项目依赖环境复杂「**近 200 个 Pod**」,正常 pod install 的过程中 **Resolving dependencies** 可以达到**小时级别**的时间，因此针对这种 Case，提供了一个一套解决方案「如果有其他业务想要使用，可以使用这个工具先打平-[BDParseLock](https://www.rubydoc.info/gems/BDParseLock/2.0.0)」,使用的话只需要传入 true 即可 PS：如果是混编大型工程的话，请使用`bytedanceAnalyzeSpeed(true,:project_type => 'swift')`

  ```ruby

  require 'bd_pod_extentions'

  bd_use_app('toutiao','thirdParty','public')

  target 'Article' do
     pod_binary 'FSCalendar', '2.7.14'
  end
  ```

  开启加速功能后，解析时间经过**gnomon**工具测试可以达到**毫秒级**。

  **Note：这个功能适用于大型业务线，如果其他业务线想要使用请先使用[BDParseLock](https://www.rubydoc.info/gems/BDParseLock/2.0.0)进行依赖解析，因为加速开启后，是不进行 dependency 解析的，因此需要将 Podfile 与 Podfile.lock 对齐**

- [3] bd_use_source!

  bd_use_source！可以将所有的 Pods 切换成源码模式

  ```ruby
  require 'bd_pod_extentions'

    bd_use_source！

  bd_use_app('toutiao','thirdParty','public')

  target 'Article' do
     pod_binary 'FSCalendar', '2.7.14'
  end
  ```

- [4] bd_use_binary!

  bd_use_binary！可以将所有的 Pods 切换成二进制模式

  ```ruby
  require 'bd_pod_extentions'

  bd_use_binary！

  bd_use_app('toutiao','thirdParty','public')

  target 'Article' do
   pod_binary 'FSCalendar', '2.7.14'
  end
  ```

- [5] pod_binary

  如果需要使用某一个组件的二进制版本时，只需要将

  ```ruby
  pod 'BDWebImage/SDAdapter','0.10.0' => pod_binary 'BDWebImage/SDAdapter','0.10.0'
  ```

  pod_binary 指令会根据后端白名单自动判断是否具有二进制版本，如果有，进行动态替换为 Binary，如果没有则会给出警告信息，随后使用源码进行集成。

  ```ruby
  ===============================================================================
  TTThemed Binary Warning:Current version:0.10.0 have no binary version
  ===============================================================================
  ```

- [6] pod_source

  如果需要使用某一个组件的源码版本时，只需要将

  ```ruby
  pod_binary 'TTThemed','0.10.0' => pod_source 'TTThemed','0.10.0'
  ```

- [7] plugin 'cocoapods-bytedance-transform'

  cocoapods-bytedance-transform 是一款 CocoaPods 的扩展插件，可一键切换 source、binary、dev 状态的模式，增量编译无需重新 pod install，详情可以参考这个—— [cocoapods-bytedance-transform](https://www.rubydoc.info/gems/cocoapods-BDTransform/4.0.0)

  ```ruby
  require 'bd_pod_extentions'

  plugin 'cocoapods-bytedance-transform'

  bd_use_binary！

  bd_use_app('toutiao','thirdParty','public')

  target 'Article' do
     pod_binary 'FSCalendar', '2.7.14'
  end
  ```
- [8] plugin 'cocoapods-bytedance-Slardar'

cocoapods-bytedance-Slardar  是一款 CocoaPods 的扩展插件，可与Sladar平台进行打通，做到自动分配crash处理人，自动提issue等功能，详细的可以产看这个 [Slardar与无线研发平台交互需求](https://bytedance.feishu.cn/space/doc/1Y8CwlwRnei9gmKUAtMfIh?from=message)

- [9] cocoapods-bytedance-DebugBinary

cocoapods-bytedance-Slardar  是一款 CocoaPods 的扩展插件，可在运行终端的情况下进行二进制调试，方便现场追溯问题，详细的可以查看这个 [Slardar与无线研发平台交互需求](https://bytedance.feishu.cn/space/doc/1Y8CwlwRnei9gmKUAtMfIh?from=message)

- [10] bytedanceRemoveAllConfuse

平台之前为一些敏感项目提供了在线组件混淆的功能 [组件平台文档](http://cloud.bytedance.net/mobile/docs/iOS/Platform/iOS%E9%A1%B9%E7%9B%AE%E6%8E%A5%E5%85%A5%E7%BB%84%E4%BB%B6%E5%8C%96%E5%B9%B3%E5%8F%B0%E4%BB%A5%E5%8F%8A%E6%B3%A8%E6%84%8F%E4%BA%8B%E9%A1%B9.html#%E5%B9%B3%E5%8F%B0%E4%BD%BF%E7%94%A8%E6%8C%87%E5%8D%97) 但是如果在项目脱敏后，需要将混淆的组件进行恢复，因此平台，提供了一键将项目中组件脱敏的功能。

- [11] bd_enable_DevelopMode

随着组件化的进程越来越快，新的研发同学的不断加入，大家在日常开发中存在多种开发模式。由于不同同学对于CocoaPods的熟悉程度不同，不同的开发模式之间常常会导致编译不过问题。同时为了配合主端代码规范性要求，我们定制了一套Pod 本地开发流程，具体可以看这个 [文档](https://bytedance.feishu.cn/space/doc/doccnLCNy1lCiEWIApMxeN#vAWQA4)

- [12] showCode!
平台提供了通用二进制的功能，虽然在使用二进制时可以加速编译，但是往往也会造成查找一个类名方法等不便，因此平台提供了在二进制时可以显示源码的功能，做到编译时使用二进制，但是可以将实现文件作为文档功能

- [12] :confuse

平台提供了Native端组件混淆的功能，比如需要混淆BDWebImage

```
   pod_source 'BDWebImage','1.0.0',:confuse => 'true'
```
然后**清理cache**，重新 **pod install** 一下就可以
对于NSClassFromString问题，可以引入 **AwesomeInjection** 组件进行处理

